#include <string.h>

#include "param.h"
#include "asap.h"

/* if "len" bytes can be put in the output buffer of "chan" */
#define NOTFIT(len,chan) (len + chan->out.curr > chan->out.size)

#define PUTC(c, chan) (chan->out.buf[chan->out.curr++] = c)
#define ADVANCE(len, chan) (chan->out.curr += len)

/* Should be a little more precise for length ... */
#define NATLENGTH(n) ((n < 10) ? 1 : MAXARLEN)

/*
 * All along this file, we are not too strict on bufferization i.e. we flush
 * before the buffer is full in order to avoid breaking entities.
 */

ASAPstatus_t    ASAPflush(chan)
    channel_t       chan;
{

    if (chan->out.curr == 0) {
	return (ASAP_SUCCESS);	/* nothing to do ... */
    }
    else {
	if (write(chan->out.fd, chan->out.buf, chan->out.curr) < 0) {
	    /* Be more clever please... ! */
	    perror("write failed in ASAPflush");
	    exit(ASAP_ESYS);
	}
	chan->out.curr = 0;
	return (ASAP_SUCCESS);
    }
}


ASAPstatus_t    ASAPsendNInt(chan, n)
    channel_t       chan;
    int             n;
{
    int             l;

    if (NOTFIT(MAXNINTLEN + 1, chan)) {
	ASAPflush(chan);
    }
    l = encodeInt(chan->out.buf + chan->out.curr, n);
    ADVANCE(l, chan);
    PUTC(SEP, chan);
    return ASAP_SUCCESS;
}


ASAPstatus_t    ASAPsendInt(chan, num, len)
    channel_t       chan;
    char           *num;
    int             len;
{

    if (NOTFIT(len + 1, chan)) {
	ASAPflush(chan);
    }
    if (NOTFIT(len + 1, chan)) {
	if (write(chan->out.fd, num, len) < 0) {
	    /* Be more clever please... ! */
	    perror("write failed in ASAPflush");
	    exit(ASAP_ESYS);
	}
	PUTC(SEP, chan);
	return ASAP_SUCCESS;
    }
    else {
	memcpy(chan->out.buf + chan->out.curr, num, len);
	ADVANCE(len, chan);
	PUTC(SEP, chan);
    }
    return ASAP_SUCCESS;
}


/*
 * We don't "truncate" binary data, So they are always written "at once".
 * 
 */
ASAPstatus_t    ASAPsendBinary(chan, bin, len)
    channel_t       chan;
    char           *bin;
    int             len;
{
    int             l;

    /* just in case ... */
    if (len <= 0) {
	fatal("trying to send binary data with negative length");
    }

    /* can we write the tag (i.e. BINTAG + length) ? */
    if (NOTFIT(MAXBINTAGLEN, chan)) {
	ASAPflush(chan);
    }
    /* So we write it... */
    PUTC(BINTAG, chan);
    l = encodeInt(chan->out.buf + chan->out.curr, len);
    ADVANCE(l, chan);
    PUTC(SEP, chan);

    if (NOTFIT(len + 1, chan)) {/* 1 for the "white space" a the end... */
	ASAPflush(chan);
	if (write(chan->out.fd, bin, len) < 0) {
	    perror("write failed in ASAPsendBinary");
	    exit(ASAP_ESYS);
	}
	PUTC(SEP, chan);
	return ASAP_SUCCESS;
    }
    else {
	memcpy(chan->out.buf + chan->out.curr, bin, len);
	ADVANCE(len, chan);
	PUTC(SEP, chan);
	return ASAP_SUCCESS;
    }
}



ASAPstatus_t    ASAPsendAttrOp(chan, op, arity, nbattrs)
    channel_t       chan;
    char           *op;
    int             arity, nbattrs;
{
    int             oplen = strlen(op);
    int             attrd = (nbattrs == 0) ? 0 : 1;
    int             arilen;
    int             attrlen;	/* 0 if no attributes, else 1 */

    if (oplen > MAXOPLEN) {
	/* it is not legal to send such a long operator */
	fatal("operator too long in ASAPsendOp");
    }
    /* "ATTRTAG"OP_ATRS_OPS_ is the "heading" of a term */
    if (NOTFIT(attrd + oplen + 1 + attrd * (MAXARLEN + 1) + MAXARLEN + 1, chan)) {
	ASAPflush(chan);
    }
    /* operator */
    if (attrd != 0) {
	PUTC(ATTRTAG, chan);
    }
    memcpy(chan->out.buf + chan->out.curr, op, oplen);
    ADVANCE(oplen, chan);
    PUTC(SEP, chan);

    /* attributes */
    if (attrd != 0) {
	attrlen = encodeInt(chan->out.buf + chan->out.curr, nbattrs);
	ADVANCE(attrlen, chan);
	PUTC(SEP, chan);
    }
    /* arity */
    arilen = encodeInt(chan->out.buf + chan->out.curr, arity);
    ADVANCE(arilen, chan);
    PUTC(SEP, chan);
    return ASAP_SUCCESS;
}


ASAPstatus_t    ASAPsendOp(chan, op, arity)
    channel_t       chan;
    char           *op;
    int             arity;
{
    return (ASAPsendAttrOp(chan, op, arity, 0));
}


ASAPstatus_t    ASAPsendAttrName(chan, attr)
    channel_t       chan;
    char           *attr;
{

    int             l = strlen(attr);

    if (NOTFIT(l + 1, chan)) {
	ASAPflush(chan);
    }
    memcpy(chan->out.buf + chan->out.curr, attr, l);
    ADVANCE(l, chan);
    PUTC(SEP, chan);
    return ASAP_SUCCESS;
}


/*
 * A convenient function to send an operator with no attributes and no sons.
 */
ASAPstatus_t    ASAPsendSymbol(chan, symb)
    channel_t       chan;
    char           *symb;
{
    return (ASAPsendOp(chan, symb, 0));
}

/*
 * To send an operator without a know arity.
 */
ASAPstatus_t    ASAPsendListOp(chan, op)
    channel_t       chan;
    char           *op;
{
    ASAPsendAttrOp(chan, op, 0, 1);
    ASAPsendAttrName(chan, ATTR_BEGIN_NAME);
    /* dummy value of ATTR_BEGIN */
    ASAPsendNInt(chan, 0);
    return ASAP_SUCCESS;
}

/* to end the stream of arguments */
ASAPstatus_t    ASAPsendEndOp(chan)
    channel_t       chan;
{
    ASAPsendAttrOp(chan, "_", 0, 1);
    ASAPsendAttrName(chan, ATTR_END_NAME);
    /* dummy */
    ASAPsendNInt(chan, 0);
    return (ASAP_SUCCESS);
}

ASAPstatus_t    ASAPsendPredefAttr(chan, attr)
    channel_t       chan;
    ASAPpred_t      attr;
{
    return (ASAPsendAttrName(chan, ASAPpredefName(attr)));
}
