/*
 * This file contains the functions to be used by an ASAP server.
 * "ASAPLocalConnect" and "ASAPRemoteConnect" are used to initiate the
 * connection with the client at the given address. The return type is "void"
 * because it is a fatal error not to be able to connect. WARNING : we
 * connect the "normal" socket first, so the client should always listen in
 * the same order...
 */

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>

#include "asap.h"

/* number of attempts to connect */
#define MAXTRY 300



/*
 * UNIX domain connection returns "conn". "socketname" is suffixed with "u"
 * and "n" to obtain the "normal" and "urgent" addresses.
 */
ASAPstatus_t    ASAPlocalConnect(socketname, conn)
    char           *socketname;
    ASAPconn_t     *conn;
{
    int             i;
    int             nsd, usd;
    struct sockaddr_un nad, uad;

    nsd = socket(AF_UNIX, SOCK_STREAM, 0);	/* the "normal" channel */
    usd = socket(AF_UNIX, SOCK_STREAM, 0);	/* the "urgent" channel */

    if ((nsd < 0) || (usd < 0)) {
	perror("cannot get socket descriptor in ASAPLocalCconnect");
	exit(2);
    }

    /* to be sure everything is clean... */
    bzero((char *) &nad, sizeof(nad));
    bzero((char *) &uad, sizeof(uad));

    nad.sun_family = AF_UNIX;
    uad.sun_family = AF_UNIX;

    strcpy(nad.sun_path, socketname);
    strcpy(uad.sun_path, socketname);
    strcat(nad.sun_path, "n");
    strcat(uad.sun_path, "u");

    /*
     * Now, we try to connect ... Act as a delay loop thanks to the latency
     * of the system call...
     */
    for (i = 1; (i <= MAXTRY) && (connect(nsd, &nad, sizeof nad) < 0); i++);	/* VOID */
    if (i > MAXTRY)
	fatal("cannot connect to client (normal)");
    for (i = 1; (i <= MAXTRY) && (connect(usd, &uad, sizeof uad) < 0); i++);	/* VOID */
    if (i > MAXTRY)
	fatal("cannot connect to client (urgent)");

    conn->urgent = (channel_t) ASAPalloc(sizeof(struct channel_struct));
    conn->normal = (channel_t) ASAPalloc(sizeof(struct channel_struct));

    initConn(conn, nsd, usd, (char *) 0);

    return ASAP_SUCCESS;
}

/* INTERNET domain connection */
ASAPstatus_t    ASAPremoteConnect(machine, port, conn)
    int             port;
    char           *machine;
    ASAPconn_t     *conn;
{
    int             nsd, usd;
    struct sockaddr_in nad, uad;
    int             i;

    nsd = socket(AF_INET, SOCK_STREAM, 0);
    usd = socket(AF_INET, SOCK_STREAM, 0);

    if ((nsd < 0) || (usd < 0)) {
	perror("cannot get socket descriptor in ASAPconnect");
	exit(2);
    }

    /* to be sure everything is clean... */
    bzero((char *) &nad, sizeof(nad));
    bzero((char *) &uad, sizeof(uad));

    nad.sin_family = AF_INET;
    uad.sin_family = AF_INET;
    {
	unsigned long   addr;

	/* if we are lucky, the machine is XXX.XXX.XXX.XXX ... */
	if ((addr = inet_addr(machine)) == -1) {
	    struct hostent *gethostbyname(), *host;

	    if ((host = gethostbyname(machine)) == (struct hostent *) NULL) {
		fatal("unable to get host address");
	    }
	    bcopy(host->h_addr, (char *) &nad.sin_addr, host->h_length);
	    bcopy(host->h_addr, (char *) &uad.sin_addr, host->h_length);
	}
	else {
	    bcopy(&addr, (char *) &nad.sin_addr, sizeof addr);
	    bcopy(&addr, (char *) &uad.sin_addr, sizeof addr);
	}

    }

    nad.sin_port = htons(port);
    uad.sin_port = htons(port + 1);

    for (i = 1; (i <= MAXTRY) && (connect(nsd, &nad, sizeof nad) < 0); i++);	/* VOID */
    if (i > MAXTRY)
	fatal("cannot connect to client (normal)");
    for (i = 1; (i <= MAXTRY) && (connect(usd, &uad, sizeof uad) < 0); i++);	/* VOID */
    if (i > MAXTRY)
	fatal("cannot connect to client (urgent)");

    initConn(conn, nsd, usd, (char *) 0);

    return ASAP_SUCCESS;
}
